/* handle options passed during program execution */

// note3 from shade.c: throughout the source, the terms 'options', 'switches', and 'parameters' are used interchangably

#include <stdio.h>
#include <stdlib.h> /* for atoi() or malloc() */
#include <string.h> /* for strlen() or strcpy() or strcmp() */
#include "./strint.h"
#include "./options.h"

OPTION options;
VALID_OPTION vldopts;

void initValidOpts(void)
{
  uint i=0;
  vldopts.opt[i]='r'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='w'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='e'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='p'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='m'; vldopts.args[i]=0; i++;
  vldopts.opt[i]='o'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='c'; vldopts.args[i]=0; i++;
  vldopts.opt[i]='l'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='b'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='B'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='u'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='s'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='h'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='t'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='f'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='R'; vldopts.args[i]=2; i++;
  vldopts.opt[i]='i'; vldopts.args[i]=1; i++;
  vldopts.opt[i]='T'; vldopts.args[i]=2; i++;
  vldopts.opt[i]='E'; vldopts.args[i]=0; i++;
  vldopts.opt[i]='y'; vldopts.args[i]=0; i++;
  vldopts.opt[i]='C'; vldopts.args[i]=0; i++;
  vldopts.opt[i]='x'; vldopts.args[i]=0; i++;
  vldopts.opt[i]='k'; vldopts.args[i]=0; i++;
  vldopts.opt[i]='S'; vldopts.args[i]=1; i++;
  
 return;
}

/** return the index of an option, or -1 if not found **/
int getOpt(char opt)
{
  int i=0;

  extern OPTION options;

  for (i=0; i<options.cnt; i++)
  {
    if (options.opt[i]==opt)
      return i;
  }
  return -1;
}

void initOpts(int args, char **arg)
{
  uint i=0, c=0, d=0;
  char temp[10];

  extern OPTION options;
  extern VALID_OPTION vldopts;

  options.cnt=0;
  options.opt = (char*)malloc(args);
  options.index = (int*)malloc(args*sizeof(int*));
  options.arg = (char***)malloc(args * sizeof(char**));
//  for (i=0; i<args; i++)
//    options.arg[i] = (char**)malloc(args * sizeof(char*));

  /** check options and parameters **/
  for (i=1; i<args; i++) /* skip arg[0] */
  {
    /* how many dashes were supplied? */
    d=0;
    if (arg[i][0] == '-')
    {
      d=1;
      if (arg[i][1] == '-')
        d=2;
    }

    /* get the option and put it in "temp" without any dashes */
    mid(arg[i], d, strlen(arg[i])-d, temp);

    /* make sure it was a valid option */
    for (c=0; c<NUM_OPTS; c++)
    {
      if (d=1 && strlen(temp) == 1) /* if its a simple switch (ie, '-r')*/
      {
        if (temp[0] == vldopts.opt[c])
          break;
      }
// BASH-syntax compliance .... not neccessary, and incomplete.
//      else if (d==2) /* if its a bash-syntax switch (ie, '--read')*/
//      {
//        if (strcmp(temp, vldopts.bash[c])==0)
//          break;
//      }
      if (c+1==NUM_OPTS) /* if this is was LAST possibility */
      {
        printf("ERROR: Unrecognized option: %s\n", arg[i]);
        endProgram(0);
      }
    }

    /* get switch, index, and malloc enough room for arguments */
    options.opt[options.cnt]=vldopts.opt[c];
    options.index[options.cnt]=c;
    options.arg[options.cnt] = (char**)malloc(vldopts.args[c] * sizeof(char*));

    /* if there are supposed to be arguments... */
    if (vldopts.args[c] > 0)
    {
      /* ...then get them */
      for (d=0; d<vldopts.args[c]; d++) /* recycling the d variable */
      {
        i++; /* we shouldn't loop over this again anyhow */
        if (i<args) /* not end of array, are we? */
        {
          options.arg[options.cnt][d] = (char*)malloc((strlen(arg[i])+2)*sizeof(char)); /* make room for argument */ //fixme
          strcpy(options.arg[options.cnt][d], arg[i]);
        }else{
          printf("ERROR: Missing argument for: %s\n", arg[i-(d+1)]); /* d+1 = how many args past the switch we've gone (+1 = avoid fencepost error) */

          /* free the stuff we just did and exit */
          for (d=0; c<vldopts.args[c]; d++) /* recycling 'd' variable */
            free(options.arg[d]);

          endProgram(0);
        }
      }
    }else{ /* null the arg to be safe */
      options.arg[options.cnt][c] = (char*)malloc(1);
      options.arg[options.cnt][c] = '\0';
    }

    options.cnt++;
  }

// DEBUG STUFF
//  for (i=0; i<options.cnt; i++)
//  {
//    printf("OPTION: %c ", options.opt[i]);
//
//    for (c=0; c<vldopts.args[options.index[i]]; c++)
//      printf("%s ", options.arg[i][c]);
//    printf("\n");
//  }
//  endProgram(0);
  return;
}

void freeOpts(void)
{
  int i=0, c=0;

  extern OPTION options;
  extern VALID_OPTION vldopts;

  for (i=0; i<options.cnt; i++)
  {
    for (c=0; c<vldopts.args[options.index[i]]; c++)
      free(options.arg[i][c]);

    free(options.arg[i]);
  }

  free(options.index);
  free(options.arg);
  free(options.opt);

  return;
}
